from rich import print


def input_yn(question, default=True):
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default:
        prompt = " [Y/n] "
    else:
        prompt = " [y/N] "

    while True:
        print(question + prompt, end="")
        choice = input().lower()
        if default is not None and choice == "":
            return default
        elif choice in valid:
            return valid[choice]
        else:
            print("Please respond with 'yes' or 'no' (or 'y' or 'n').")


def print_error(msg, newline, exit_after):
    print(f"[red]\u274c ERROR: {msg}", end="\n" if newline else "")

    if exit_after:
        exit(1)


def print_warning(msg, newline=True):
    print(f"[yellow]\u26a0 Warning: {msg}", end="\n" if newline else "")


def print_success(msg, newline=True):
    print(f"[green]\u2713 {msg}", end="\n" if newline else "")



SENTRY_SDK_URL = (
    "https://0da75bab4671455ea1b7580cb93649f5@o444286.ingest.sentry.io" "/5645833"
)

NO_CONFIGURATION_WARNING_MSG = """\
No configuration file found or configuration is invalid. You need to configure shippy
before you can start using it.
"""

PRERELEASE_WARNING_MSG = """\
You're running a prerelease build of shippy. Be careful as prerelease versions can
behave in unexpected ways! If you haven't been instructed to test shippy, please
consider switching back to a stable build.
"""

SERVER_COMPAT_ERROR_MSG = """\
The server you're connecting to is out-of-date.
If you know the server admin, please ask them to upgrade the server.
 * Reported server version: \t{}
 * Compatible version: \t\t{}

To prevent data corruption, shippy will not work with an outdated server. Exiting...
"""

SHIPPY_COMPAT_ERROR_MSG = """\
Error: shippy is out-of-date and will not with this server instance.
Upgrade shippy with the following command:
\tpip3 install --upgrade shipper-shippy

Version information:
 * Reported compatible version by server: \t{}
 * Your current shippy version: \t\t{}
"""

SHIPPY_OUTDATED_MSG = """\
Warning: shippy is out-of-date.
 * Current version: \t{}
 * New version: \t{}

We recommend updating shippy with the following command:
\tpip3 install --upgrade shipper-shippy
"""

RATE_LIMIT_MSG = "shippy has been rate limited."
RATE_LIMIT_WAIT_STATUS_MSG = (
    "Waiting to resume after being rate limited. shippy will "
    "resume uploading in {} seconds."
)

WAITING_FINALIZATION_MSG = (
    "Waiting for the server to process the uploaded build. "
    "This may take around 30 seconds..."
)

CANNOT_CONTACT_SERVER_ERROR_MSG = "Cannot contact the server. "
UNEXPECTED_SERVER_RESPONSE_ERROR_MSG = "The server returned an unexpected response. "
FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG = (
    "Failed to retrieve server version information! "
)
FAILED_TO_LOG_IN_ERROR_MSG = "Failed to log into server! "
UNKNOWN_UPLOAD_START_ERROR_MSG = "Something went wrong starting the upload."
UNKNOWN_UPLOAD_ERROR_MSG = "Something went wrong during the upload."

UNHANDLED_EXCEPTION_MSG = """\
shippy crashed for an unknown reason. :(
To figure out what went wrong, please pass along the full output.
----
URL of request: {}
Request response code: {}
Request response: {}
---
"""

# Chunk size MUST be at 1 MB, as nginx defaults limit request size to that (or less)
CHUNK_SIZE = 1000000



import hashlib
import json
import os.path
import requests
import re
import time
import urllib.parse

from json.decoder import JSONDecodeError

import semver
from rich.console import Console
from rich.progress import (
    BarColumn,
    DownloadColumn,
    Progress,
    TextColumn,
    TimeRemainingColumn,
    TransferSpeedColumn,
)
from loguru import logger

from .constants import (
    UNHANDLED_EXCEPTION_MSG,
    FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG,
    RATE_LIMIT_WAIT_STATUS_MSG,
    RATE_LIMIT_MSG,
    UNKNOWN_UPLOAD_ERROR_MSG,
    UNKNOWN_UPLOAD_START_ERROR_MSG,
    WAITING_FINALIZATION_MSG,
    CHUNK_SIZE,
)
from .exceptions import LoginException, UploadException
from .version import __version__
from .server_compat_version import server_compat_version

console = Console()

# Set up progress bar
progress = Progress(
    TextColumn("[progress.description]{task.description}"),
    BarColumn(),
    "[progress.percentage]{task.percentage:>3.1f}%",
    "•",
    DownloadColumn(),
    "•",
    TransferSpeedColumn(),
    "•",
    TimeRemainingColumn(),
    transient=True,
)


def log_debug_request_send(request_type, url, headers=None, data=None):
    logger.debug(
        f"Sending {request_type} request to {url}, with header {headers} and "
        f"data {data}"
    )


def log_debug_request_response(r):
    try:
        r_content = r.json()
    except ValueError:
        r_content = r.content

    logger.debug(f"Received response: {r_content}")


class Client:
    def __init__(self, server_url, token=None):
        self.server_url = server_url
        self.token = token

    def is_url_secure(self):
        return self.server_url[0:5] == "https"

    def is_server_compatible(self):
        server_compat = semver.VersionInfo.parse(server_compat_version)
        return self.get_version() >= server_compat

    def is_shippy_compatible(self):
        shippy_version = semver.VersionInfo.parse(__version__)
        return shippy_version >= self.get_shippy_compat_version()

    def login(self, username, password):
        r = self._post(
            url="/api/v1/maintainers/login/",
            data={"username": username, "password": password},
        )

        match r.status_code:
            case 200:
                token = r.json()["token"]

                if token == b"":
                    raise LoginException("Server returned an empty token.")
                else:
                    self.token = token
            case 301:
                if not self.is_url_secure():
                    raise LoginException(
                        "Server uses HTTPS, but was supplied HTTP URL."
                    )
            case 400:
                if r.json()["error"] == "blank_username_or_password":
                    raise LoginException("Username or password must not be blank.")
            case 404:
                if r.json()["error"] == "invalid_credential":
                    raise LoginException("Invalid credentials!")
            case 502:
                raise LoginException("The gateway server is currently unavailable.")
            case 503:
                raise LoginException("The server is temporarily unavailable.")
            case _:
                handle_undefined_response(r)

    def get_version(self):
        return semver.VersionInfo.parse(self._get_info()["version"])

    def get_shippy_compat_version(self):
        return semver.VersionInfo.parse(self._get_info()["shippy_compat_version"])

    def get_shippy_upload_variants(self):
        return json.loads(self._get_info()["shippy_upload_variants"])

    def _get_info(self):
        r = self._get(url="/api/v1/system/info")
        if r.status_code == 200:
            return r.json()
        else:
            raise Exception(FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG)

    def get_regex_pattern(self):
        r = self._get(
            url="/api/v1/maintainers/upload_filename_regex_pattern",
            headers=self._get_header(),
        )

        if r.status_code == 200:
            return r.json()["pattern"]

    def _get_checksum_type(self):
        return self._get_info()["shippy_upload_checksum_type"]

    def get_username(self):
        r = self._get(
            url="/api/v1/maintainers/token_check/",
            headers=self._get_header(),
        )

        return r.json()["username"]

    def is_token_valid(self):
        r = self._get(
            url="/api/v1/maintainers/token_check/",
            headers=self._get_header(),
        )

        return r.status_code == 200

    def _get_upload_info(self, build_path):
        current_byte = 0
        upload_id = ""

        # Check for previous attempts
        try:
            previous_attempts = self._get(
                url="/api/v1/maintainers/chunked_upload/", headers=self._get_header()
            ).json()
        except requests.exceptions.RequestException:
            raise UploadException(UNKNOWN_UPLOAD_ERROR_MSG)
        for attempt in previous_attempts:
            if build_path == attempt["filename"]:
                logger.debug(
                    f"Found a previous upload attempt for the build {build_path}, "
                    f"created on {attempt['created_at']}",
                )
                current_byte = attempt["offset"]
                upload_id = attempt["id"]
        return current_byte, upload_id

    def upload(self, build_path):
        total_file_size = os.path.getsize(build_path)

        with progress:
            upload_progress = progress.add_task(
                "[green]Uploading...", total=total_file_size
            )

            current_byte, upload_id = self._get_upload_info(build_path)

            with open(build_path, "rb") as build_file:
                build_file.seek(current_byte)
                chunk = build_file.read(CHUNK_SIZE)
                while chunk:
                    try:
                        r = self._upload_chunk(
                            build_path=build_path,
                            chunk=chunk,
                            current=current_byte,
                            total=total_file_size,
                            upload_id=upload_id,
                        )

                        if r.status_code == 200:
                            upload_id = r.json()["id"]
                            current_byte += len(chunk)
                            progress.update(upload_progress, completed=current_byte)

                            # Read next chunk and continue
                            chunk = build_file.read(CHUNK_SIZE)
                        elif int(r.status_code / 100) == 4:
                            upload_handle_4xx_response(r)
                        else:
                            raise UploadException(UNKNOWN_UPLOAD_START_ERROR_MSG)
                    except requests.exceptions.RequestException:
                        raise UploadException(UNKNOWN_UPLOAD_ERROR_MSG)

        # Finalize upload to begin processing
        try:
            with console.status(WAITING_FINALIZATION_MSG):
                checksum = get_hash_of_file(
                    build_path, checksum_type=self._get_checksum_type()
                )
                r = self._upload_finalize(upload_id=upload_id, checksum=checksum)

                upload_exception_check(r, build_path)
        except UploadException as e:
            raise e
        except requests.exceptions.RequestException:
            raise UploadException(UNKNOWN_UPLOAD_ERROR_MSG)

        return upload_id

    def disable_build(self, upload_id):
        r = self._post(
            "/api/v1/maintainers/build/enabled_status_modify/",
            headers=self._get_header(),
            data={"build_id": upload_id, "enable": False},
        )

        if r.status_code == 200:
            print(f"Build {upload_id} has been disabled.")
        else:
            raise Exception("There was a problem disabling the build.")

    def _upload_chunk(self, build_path, chunk, current, total, upload_id):
        if upload_id:
            url = f"/api/v1/maintainers/chunked_upload/{upload_id}/"
        else:
            url = "/api/v1/maintainers/chunked_upload/"
        result = self._put(
            url=url,
            headers=self._get_header(chunk=chunk, current=current, total=total),
            data={"filename": build_path},
            files={"file": chunk},
        )
        logger.debug(f"Got back: {result}")
        return result

    def _upload_finalize(self, upload_id, checksum):
        return self._post(
            url=f"/api/v1/maintainers/chunked_upload/{upload_id}/",
            headers=self._get_header(),
            data={self._get_checksum_type(): checksum},
        )

    def _get_header(self, chunk=None, current=None, total=None):
        header = {
            "User-Agent": f"shippy {__version__}",
            "Authorization": f"Token {self.token}",
        }

        if chunk is not None and current is not None and total is not None:
            header[
                "Content-Range"
            ] = f"bytes {current}-{current + len(chunk) - 1}/{total}"

        return header

    def _request(self, type, url, headers=None, data=None, files=None):
        request_url = urllib.parse.urljoin(self.server_url, url)
        log_debug_request_send(
            request_type=type,
            url=request_url,
            headers=headers,
            data=data,
        )
        match type:
            case "GET":
                r = requests.get(url=request_url, headers=headers, data=data)
            case "POST":
                r = requests.post(
                    url=request_url,
                    headers=headers,
                    data=data,
                    allow_redirects=False,
                )
            case "PUT":
                r = requests.put(
                    url=request_url,
                    headers=headers,
                    data=data,
                    files=files,
                )
            case _:
                return
        log_debug_request_response(r)

        # Check for rate limit
        if r.status_code == 429:
            print(RATE_LIMIT_MSG)
            self._wait_rate_limit(int(re.findall(r"\d+", r.json()["detail"])[0]))

            return self._request(
                type=type, url=url, headers=headers, data=data, files=files
            )

        return r

    def _wait_rate_limit(seconds):
        with console.status(RATE_LIMIT_WAIT_STATUS_MSG.format(seconds)) as status:
            while seconds:
                time.sleep(1)
                seconds -= 1
                status.update(status=RATE_LIMIT_WAIT_STATUS_MSG.format(seconds))

    def _post(self, url, headers=None, data=None):
        return self._request("POST", url, headers, data)

    def _get(self, url, headers=None, data=None):
        return self._request("GET", url, headers, data)

    def _put(self, url, headers, data, files):
        return self._request("PUT", url, headers, data, files)


def handle_undefined_response(request):
    """Handles undefined responses sent back by the server"""
    try:
        raise Exception(
            UNHANDLED_EXCEPTION_MSG.format(
                request.url, request.status_code, request.json()
            )
        )
    except JSONDecodeError:
        raise Exception(
            UNHANDLED_EXCEPTION_MSG.format(
                request.url, request.status_code, request.content
            )
        )


def upload_handle_4xx_response(chunk_request):
    try:
        response_json = chunk_request.json()
        raise UploadException(response_json["message"])
    except (KeyError, JSONDecodeError):
        raise UploadException(UNKNOWN_UPLOAD_ERROR_MSG)


def get_hash_of_file(filename, checksum_type):
    if checksum_type.lower() == "md5":
        hash_obj = hashlib.md5()
    elif checksum_type.lower() == "sha256":
        hash_obj = hashlib.sha256()
    else:
        # Unsupported checksum type
        return None

    with open(filename, "rb") as file:
        content = file.read()
        hash_obj.update(content)
    return hash_obj.hexdigest()


def get_hash_from_checksum_file(checksum_file):
    with open(checksum_file, "r") as checksum_file_raw:
        line = checksum_file_raw.readline()
        values = line.split(" ")
        return values[0]


def find_checksum_file(filename):
    valid_checksum_types = ["md5", "sha256"]
    has_checksum_file_type = None
    has_sum_postfix = False
    for checksum_type in valid_checksum_types:
        if os.path.isfile(f"{filename}.{checksum_type}"):
            has_checksum_file_type = checksum_type
            has_sum_postfix = False
        elif os.path.isfile(f"{filename}.{checksum_type}sum"):
            has_checksum_file_type = checksum_type
            has_sum_postfix = True
    return has_checksum_file_type, has_sum_postfix


def upload_exception_check(request, build_file):
    if request.status_code == 200:
        print(f"Successfully uploaded the build {build_file}!")
        return
    elif int(request.status_code / 100) == 4:
        try:
            response_json = request.json()
            raise UploadException(response_json["message"])
        except (JSONDecodeError, KeyError):
            raise UploadException("An unknown error occurred parsing the response.")
    elif int(request.status_code / 100) == 5:
        raise UploadException(
            "An internal server error occurred. Please contact the admins."
        )

    handle_undefined_response(request)



import argparse
import re
import glob
import os.path
import signal
import sys
from json import JSONDecodeError
from loguru import logger

import requests
import semver
import sentry_sdk

from rich import print
from rich.console import Console

from .client import (
    get_hash_from_checksum_file,
    get_hash_of_file,
    find_checksum_file,
    Client,
)
from .config import get_config_value, set_config_value, get_optional_true_config_value
from .constants import (
    SENTRY_SDK_URL,
    SERVER_COMPAT_ERROR_MSG,
    SHIPPY_COMPAT_ERROR_MSG,
    SHIPPY_OUTDATED_MSG,
    UNEXPECTED_SERVER_RESPONSE_ERROR_MSG,
    FAILED_TO_LOG_IN_ERROR_MSG,
    CANNOT_CONTACT_SERVER_ERROR_MSG,
    PRERELEASE_WARNING_MSG,
    NO_CONFIGURATION_WARNING_MSG,
)
from .exceptions import LoginException, UploadException
from .helper import input_yn, print_error, print_warning, print_success
from .version import __version__
from .server_compat_version import server_compat_version

sentry_sdk.init(
    SENTRY_SDK_URL,
    traces_sample_rate=1.0,
    release=__version__,
    ignore_errors=[ConnectionError],
)

console = Console()


# Handle SIGINT gracefully and don't puke up traceback
def sigint_handler(*_):
    exit(1)


signal.signal(signal.SIGINT, sigint_handler)


def lower_logger_level():
    logger.remove()
    logger.add(sys.stderr, level="INFO")


def main():
    # Get commandline arguments
    args = init_argparse()

    lower_logger_level()

    if args.version:
        print(__version__)
        return

    if args.debug:
        print_warning("Debug mode has been turned on!")
        logger.add(sink="shippy_{time}.log", level="DEBUG", enqueue=True)

    print(f"Welcome to shippy (v.{__version__})!")

    # Check for updates
    check_shippy_update()

    # Initialize client
    client = build_client_from_config()
    server_prechecks(client)

    # Start uploads
    search_and_upload_builds(client, args)


def server_prechecks(client):
    check_server_compat(client)
    check_token_validity(client)


def check_and_upload_build(client, args, build_path):
    # Check build file validity
    if not check_build(client, build_path):
        print_warning("Invalid build. Skipping...")
        return

    if is_upload_without_prompt_enabled(args) or input_yn(
        f"Uploading build {build_path}. Start?"
    ):
        try:
            upload_id = client.upload(build_path=build_path)

            if is_build_disabling_enabled():
                client.disable_build(upload_id=upload_id)
        except UploadException as exception:
            print_error(exception, newline=True, exit_after=False)


def search_and_upload_builds(client, args):
    # Search current directory for files with regex pattern returned by server
    build_paths = get_builds_in_current_dir(client.get_regex_pattern())

    if len(build_paths) == 0:
        print_error(
            msg="No files matching the submission criteria were detected in the "
            "current directory.",
            newline=True,
            exit_after=False,
        )
    else:
        print(f"Detected {len(build_paths)} build(s):")
        for build_path in build_paths:
            print(f"\t{build_path}")

        if not is_upload_without_prompt_enabled(args) and len(build_paths) > 1:
            print_warning("You seem to be uploading multiple builds. ", newline=False)
            if not input_yn("Are you sure you want to continue?", default=False):
                return

        for build_path in build_paths:
            check_and_upload_build(client, args, build_path)


def is_upload_without_prompt_enabled(args):
    config_value = get_optional_true_config_value("shippy", "UploadWithoutPrompt")

    return config_value or args.yes


def build_client_from_config():
    try:
        url = get_config_value("shippy", "server")
        if not check_server_url_schema(url):
            print_error(
                msg="The configuration file is corrupt. Please delete it and restart "
                "shippy.",
                newline=True,
                exit_after=True,
            )

        token = get_config_value("shippy", "token")
        server = Client(server_url=url, token=token)
    except KeyError:
        print_warning(NO_CONFIGURATION_WARNING_MSG)
        server = Client(server_url=get_server_url())
        prompt_login(server)
    return server


def init_argparse():
    parser = argparse.ArgumentParser(
        description="Client-side tool for interfacing with shipper"
    )
    parser.add_argument(
        "-y",
        "--yes",
        action="store_true",
        help="Upload builds automatically without prompting",
    )
    parser.add_argument(
        "-d",
        "--debug",
        action="store_true",
        help="Enable debug mode",
    )
    parser.add_argument(
        "-v",
        "--version",
        action="store_true",
        help="Show version of shippy and exit",
    )
    return parser.parse_args()


def check_server_compat(client):
    with console.status(
        "Please wait while shippy contacts the remote server to check compatibility... "
    ):
        # Check if shipper version is compatible
        if not client.is_server_compatible():
            print_error(
                msg=SERVER_COMPAT_ERROR_MSG.format(
                    client.get_version(), server_compat_version
                ),
                newline=True,
                exit_after=True,
            )

    # Check if shippy version is compatible, but only if running stable builds
    if is_prerelease():
        print_warning(
            "You're running a prerelease build of shippy. Server compatibility checks "
            "are disabled."
        )
    else:
        if not client.is_shippy_compatible():
            print_error(
                msg=SHIPPY_COMPAT_ERROR_MSG.format(
                    client.get_shippy_compat_version(), __version__
                ),
                newline=True,
                exit_after=True,
            )

    print_success("Finished compatibility check. No problems found.")


def check_token_validity(client):
    with console.status(
        "Please wait while shippy contacts the remote server to check if the token is "
        "still valid... "
    ):
        if client.is_token_valid():
            print_success(
                f"Successfully validated token! Hello, {client.get_username()}!"
            )
        else:
            # Token check failed, prompt for login again
            print_warning("The saved token is invalid. Please sign-in again.")
            prompt_login(client)


def check_shippy_update():
    with console.status("Please wait while shippy checks for updates... "):
        try:
            r = requests.get(
                "https://api.github.com/repos/shipperstack/shippy/releases/latest"
            )
            latest_version = r.json()["name"]
        except KeyError:
            print_error(
                "Failed to contact the GitHub API to check the latest version.",
                newline=True,
                exit_after=False,
            )

    # Check if user is running an alpha/beta build
    if is_prerelease():
        print_warning(PRERELEASE_WARNING_MSG)
    else:
        # User is running a stable build, proceed with update check
        if semver.compare(__version__, latest_version) == -1:
            print(SHIPPY_OUTDATED_MSG.format(__version__, latest_version))
        else:
            print_success("Finished update check. shippy is up-to-date!")


def is_prerelease():
    return "a" in __version__ or "b" in __version__


def get_builds_in_current_dir(regex_pattern):
    with console.status("Detecting builds in current directory..."):
        builds = []
        files = [f for f in glob.glob("*.zip")]
        for file in files:
            if re.search(regex_pattern, file):
                builds.append(file)

        return builds


def check_build(client, filename):
    """Makes sure the build is valid"""
    print(f"Validating build {filename}...")
    # Validate that there is a matching checksum file
    has_checksum_file_type, has_sum_postfix = find_checksum_file(filename=filename)

    if has_checksum_file_type is None:
        print_warning(
            "This build does not have a matching checksum file. ", newline=False
        )
        return False
    else:
        print_success("Matching checksum file exists!")

    # Validate checksum
    with console.status(f"Checking {has_checksum_file_type.upper()} hash..."):
        hash_val = get_hash_of_file(
            filename=filename, checksum_type=has_checksum_file_type
        )
        if not has_sum_postfix:
            actual_hash_val = get_hash_from_checksum_file(
                f"{filename}.{has_checksum_file_type}"
            )
        else:
            actual_hash_val = get_hash_from_checksum_file(
                f"{filename}.{has_checksum_file_type}sum"
            )
        if hash_val != actual_hash_val:
            print_error(
                msg="This build's checksum is invalid. ",
                newline=False,
                exit_after=False,
            )
            return False
        print_success(f"Checksum {has_checksum_file_type.upper()} hash matches!")

    build_slug, _ = os.path.splitext(filename)
    _, _, _, build_type, build_variant, _ = build_slug.split("-")

    # Check build type
    if build_type != "OFFICIAL":
        print_error(msg="This build is not official. ", newline=False, exit_after=False)
        return False
    else:
        print_success("Build type is official!")

    # Check build variant
    if build_variant not in client.get_shippy_upload_variants():
        print_error(
            msg="This build has an unknown variant. ",
            newline=False,
            exit_after=False,
        )
        return False
    else:
        print_success(f"Build variant {build_variant} is supported!")

    print_success(f"Validation of build {filename} complete. No problems found.")
    return True


def get_server_url():
    try:
        while True:
            server_url = input("Enter the server URL: ")
            if not check_server_url_schema(server_url):
                # noinspection HttpUrlsUsage
                print_error(
                    msg="Server URL is missing either http:// or https://.",
                    newline=True,
                    exit_after=False,
                )
            else:
                break

        set_config_value("shippy", "server", server_url)

        return server_url
    except KeyboardInterrupt:
        exit(0)


def check_server_url_schema(url):
    return "http" in url


def prompt_login(client):
    while True:
        from getpass import getpass

        try:
            username = input("Enter your username: ")
            password = getpass(prompt="Enter your password: ")

            try:
                client.login(username=username, password=password)
                set_config_value("shippy", "token", client.token)
                print_success("Successfully logged in!")
                return
            except LoginException as exception:
                print_error(exception, newline=True, exit_after=True)
            except JSONDecodeError:
                print_error(
                    msg=UNEXPECTED_SERVER_RESPONSE_ERROR_MSG
                    + FAILED_TO_LOG_IN_ERROR_MSG,
                    newline=True,
                    exit_after=True,
                )
            except requests.exceptions.RequestException as e:
                logger.error(e)
                print_error(
                    msg=CANNOT_CONTACT_SERVER_ERROR_MSG + FAILED_TO_LOG_IN_ERROR_MSG,
                    newline=True,
                    exit_after=True,
                )
        except KeyboardInterrupt:
            exit(0)


def is_build_disabling_enabled():
    try:
        return get_config_value("shippy", "DisableBuildOnUpload") == "true"
    except KeyError:
        return False


if __name__ == "__main__":
    main()
